package de.intevation.artifactdatabase.rest;

import org.restlet.resource.Get;  
import org.restlet.resource.Post;  
import org.restlet.resource.ServerResource;

import org.restlet.representation.Representation;
import org.restlet.representation.EmptyRepresentation;

import org.restlet.ext.xml.DomRepresentation;

import de.intevation.artifacts.ArtifactDatabase;
import de.intevation.artifacts.Artifact;

import org.restlet.data.Request;
import org.restlet.data.Response;
import org.restlet.data.Status;

import org.restlet.data.MediaType;

import org.apache.log4j.Logger;

import org.w3c.dom.Document;

import java.io.IOException;

import de.intevation.artifacts.ArtifactNamespaceContext;

import de.intevation.artifactdatabase.XMLUtils;

/**
 * @author Sascha L. Teichmann (sascha.teichmann@intevation)
 */
public class ArtifactResource
extends      ServerResource
{
    private static Logger logger = Logger.getLogger(ArtifactResource.class);

    public static final String XPATH_ACTION = "/action/type/@name";

    public static final String PATH = "/artifact/{uuid}";

    public static final String NO_ACTION_MESSAGE      = "no action given";
    public static final String NO_SUCH_ACTION_MESSAGE = "no such action";

    public static final String NO_ARTIFACT_FOUND = "Artifact not found";

    public static final String ADVANCE = "advance";
    public static final String FEED    = "feed";

    @Get
    public Representation represent() {

        Request request = getRequest();

        String identifier = (String)request.getAttributes().get("uuid");

        if (logger.isDebugEnabled()) {
            logger.debug("looking for artifact id '" + identifier + "'");
        }

        ArtifactDatabase db = (ArtifactDatabase)getContext()
            .getAttributes().get("database");

        Artifact artifact = db.getArtifact(identifier);

        if (artifact == null) {
            Response response = getResponse();
            response.setStatus(Status.CLIENT_ERROR_NOT_FOUND, NO_ARTIFACT_FOUND);
            return new EmptyRepresentation();
        }

        Document description = artifact.describe(db.getArtifactContext());

        if (logger.isDebugEnabled()) {
            logger.debug("out document: " + description);
        }

        return new DomRepresentation(
            MediaType.APPLICATION_XML, description);
    }

    @Post
    public Representation representPost() {
        Request request = getRequest();

        Representation requestRepr = request.getEntity();

        Document inputDocument = null;
        try {
            DomRepresentation input = new DomRepresentation(requestRepr);
            inputDocument = input.getDocument();
        }
        catch (IOException ioe) {
            logger.error(ioe.getMessage());
            Response response = getResponse();
            response.setStatus(Status.CLIENT_ERROR_BAD_REQUEST, ioe);
            return new EmptyRepresentation();
        }

        String action = XMLUtils.xpathString(
            inputDocument,
            XPATH_ACTION,
            ArtifactNamespaceContext.INSTANCE);

        if (action == null || action.length() == 0) {
            Response response = getResponse();
            response.setStatus(Status.CLIENT_ERROR_BAD_REQUEST, NO_ACTION_MESSAGE);
            return new EmptyRepresentation();
        }

        int actionType = -1;

             if (FEED   .equals(action)) actionType = 0;
        else if (ADVANCE.equals(action)) actionType = 1;
        else {
            Response response = getResponse();
            response.setStatus(Status.CLIENT_ERROR_BAD_REQUEST, NO_SUCH_ACTION_MESSAGE);
            return new EmptyRepresentation();
        }

        String identifier = (String)request.getAttributes().get("uuid");

        if (logger.isDebugEnabled()) {
            logger.debug("looking for artifact id '" + identifier + "'");
        }

        ArtifactDatabase db = (ArtifactDatabase)getContext()
            .getAttributes().get("database");

        Artifact artifact = db.getArtifact(identifier);

        if (artifact == null) {
            Response response = getResponse();
            response.setStatus(Status.CLIENT_ERROR_NOT_FOUND, NO_ARTIFACT_FOUND);
            return new EmptyRepresentation();
        }

        Document document = null;

        switch (actionType) {
            case 0:
                document = artifact.feed(inputDocument, db.getArtifactContext());
                break;
            case 1:
                document = artifact.advance(inputDocument, db.getArtifactContext());
                break;
            default:
                // should not happen
                return new EmptyRepresentation();
        }

        return new DomRepresentation(MediaType.APPLICATION_XML, document);
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
